// ////////////////////////////////////////////////////
//
// StdView.cpp : implementation of the CStdView class
// Based on class COGLView : public CView
// Inspect COGLView for standard OpenGL initialization
//
////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Std.h"

#include "StdDoc.h"
#include "StdView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////
// You can put definitions, variables and helperfunctions
// here. Functions that does not rely on CStdView member
// variables or functions

//////////////////////////////////////////////////////////
// CStdView

IMPLEMENT_DYNCREATE(CStdView, COGLView)

BEGIN_MESSAGE_MAP(CStdView, COGLView)
	//{{AFX_MSG_MAP(CStdView)
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

///////////////////////////////////////////////////////////
// CStdView construction/destruction

CStdView::CStdView()
{
	m_bIsRotating=FALSE;
	m_VX=0.0f;
	m_VZ=0.0f;

	m_hRC=NULL;
	m_hDC=NULL;
};

CStdView::~CStdView()
{}

////////////////////////////////////////////////////////////
// CStdView diagnostics
// You should usually not touch these:
#ifdef _DEBUG
void CStdView::AssertValid() const
{COGLView::AssertValid();}
void CStdView::Dump(CDumpContext& dc) const
{COGLView::Dump(dc);}

CStdDoc* CStdView::GetDocument() // non-debug version is inline
{	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CStdDoc)));
	return (CStdDoc*)m_pDocument;
}
#endif //_DEBUG

//////////////////////////////////////////////////////////////
// CStdView operations

void CStdView::DrawScene()
{
	// Have made own draw instead of COGLVIEW::DrawScene()

	// Clear the color and depth buffers.
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
		
	// Set up for scene
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();

	// position the eye relative to scene
	gluLookAt(	4.0f,0.0f,0.0f,	// eye
				0.0f,0.0f,0.0f,	// looking at
				0.0f,0.0f,1.0f);// is up

	// do rotations according to mouse
	glRotatef(m_VZ,0,0,1);
	glRotatef(m_VX,1,0,0);
	
	// setMaterial kind of yellow
    GLfloat mat_ambient[] = { 0.7f, 0.7f, 0.7f, 1.0f };
    GLfloat mat_diffuse[] = { 0.8f, 0.8f, 0.0f, 1.0f };
	glMaterialfv(GL_FRONT, GL_AMBIENT, mat_ambient);
	glMaterialfv(GL_FRONT, GL_DIFFUSE, mat_diffuse);
	
	// use a quadric to draw a circle
	GLUquadricObj *glpQ;
	glpQ=gluNewQuadric();
	gluCylinder(glpQ,1,1,1,20,20);
	gluDeleteQuadric(glpQ);

	// finnish drawing
	glFlush();
}

void CStdView::InitializeRContext()
{
	// Replace this call with your own one-time 
	// initialization code
	COGLView::InitializeRContext();
}

void CStdView::SetPerspective(int cx, int cy)
{
    // We will work on the projection matrix,
	// so set matrix mode
	glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
	// set up your viewing volume
	gluPerspective(70.0f, (GLdouble)cx/cy, 0.1f, 70.0f);
	// reset matrix mode 
	glMatrixMode(GL_MODELVIEW);
}


void CStdView::OnMouseMove(UINT nFlags, CPoint point) 
{
	// if left button is down:
	// change rotation angles and update drawing 
	// if left button is down 
	if(nFlags & MK_LBUTTON)
	{	
		// and has moved a significant dist
		if((abs(m_PLast.x-point.x)>0)||(abs(m_PLast.y-point.y)>0))
		{
			// we find an intuitive rot angle
			CRect R;
			GetClientRect(&R);
			m_VZ+=180.0f*((GLfloat (point.x-m_PLast.x))/GLfloat (R.Width()));
			m_VX+=180.0f*((GLfloat (point.y-m_PLast.y))/GLfloat (R.Height()));

			m_PLast=point;

			RedrawWindow();
		}
	}
	COGLView::OnMouseMove(nFlags, point);
}

void CStdView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	// Left mousebutton down, prepare rotation
	m_PLast=point;
	SetCapture();
	
	COGLView::OnLButtonDown(nFlags, point);
}

void CStdView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	// Left mousebutton  released, end rotation
	ReleaseCapture();
	COGLView::OnLButtonUp(nFlags, point);
}
